/* Copyright (c) 2004 Alessandro Rubini. GNU GPL 2 or later */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include "sdm.h"

MODULE_LICENSE("GPL");

int sdm_size = 128; /* megs */
int sdm_npages;
int sdm_verbose = 1;
unsigned long *sdm_pages;

module_param(sdm_size, int, 0);
module_param(sdm_verbose, int, 0);

EXPORT_SYMBOL(sdm_size);
EXPORT_SYMBOL(sdm_npages);
EXPORT_SYMBOL(sdm_verbose);
EXPORT_SYMBOL(sdm_pages);

static int sdm_d_init(void)
{
	sdm_npages = sdm_size << (20 - PAGE_SHIFT);
	sdm_pages = kmalloc(sdm_npages * sizeof(unsigned long *), GFP_KERNEL);
	if (!sdm_pages) return -ENOMEM;
	memset(sdm_pages, 0, sdm_npages * sizeof(unsigned long *));
	return 0;
}

static void sdm_d_exit(void)
{
	int i;
	for (i = 0; i < sdm_npages; i++) {
		if (sdm_pages[i])
			free_page(sdm_pages[i]);
	}
	kfree(sdm_pages);
}
module_init(sdm_d_init);
module_exit(sdm_d_exit);
